<?php

namespace Drupal\layout_builder_block_headings_extra_field\Plugin\ExtraField\Display;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\extra_field_plus\Plugin\ExtraFieldPlusDisplayBase;
use Drupal\extra_field_plus\Plugin\ExtraFieldPlusDisplayInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implement Block Heading extra_field plugin.
 *
 * @ExtraFieldDisplay(
 *   id = "layout_builder_block_headings",
 *   label = @Translation("Block Heading"),
 *   bundles = {
 *     "block_content.*"
 *   }
 * )
 */
class LayoutBuilderBlockHeadingsExtraField extends ExtraFieldPlusDisplayBase implements ContainerFactoryPluginInterface, ExtraFieldPlusDisplayInterface {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RendererInterface $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected static function extraFieldSettingsForm(): array {
    $form = parent::extraFieldSettingsForm();

    $form['tag'] = [
      '#type' => 'select',
      '#title' => t('Default tag'),
      '#options' => [
        'h1' => 'h1',
        'h2' => 'h2',
        'h3' => 'h3',
        'h4' => 'h4',
        'h5' => 'h5',
        'h6' => 'h6',
      ],
      // The empty option.
      '#empty_option' => t('- None -'),
    ];

    $form['style'] = [
      '#type' => 'textfield',
      '#title' => t('Default style'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected static function defaultExtraFieldSettings(): array {
    $values = parent::defaultExtraFieldSettings();

    $values += [
      'tag' => '',
      'style' => '',
    ];

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function view(ContentEntityInterface $entity) {
    $settings = $this->getEntityExtraFieldSettings();

    $elements = [];

    $variables['elements']['#layout_builder_block_headings'] = [
      'override_heading_level' => (bool) $settings['tag'],
      'heading_level' => $settings['tag'],
      'override_heading_style' => (bool) $settings['style'],
      'heading_style' => $settings['style'],
    ];
    $variables['content']['#block_content'] = $entity;
    layout_builder_block_headings_preprocess_block($variables);

    if ($variables['heading']) {
      $title_content = $this->renderer->render($variables['heading']);
      $title = [
        '#markup' => $title_content,
      ];

      $heading_level = $variables['heading_level'] ?? $settings['tag'];

      if ($heading_level) {
        $attributes = $variables['title_attributes'] ?? ['class' => [$settings['style']]];

        $elements[] = [
          '#type' => 'html_tag',
          '#tag' => $heading_level,
          '#value' => $title_content,
          '#attributes' => $attributes,
        ];
      }
      else {
        $elements[] = $title;
      }
    }

    return $elements;
  }

}
